package org.jvisu.base;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values in a bar graph
 * <p>
 * $Id: JVBar.java,v 1.16 2004/04/19 08:20:55 tgl Exp $
 * </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * <li> 1.1 Remove the JProgressBar stuff and make it based on JVScales
 * </ul>
 * <p>
 * </p>
 *  JVBar is used to display numerical values as a bar.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.1
 */
public class JVBar extends JVScales {
    

    /** Default inset of the bar */
    protected final static int  DEFAULT_INS = 4;
    /** The Rectangle of the bar */
    protected Rectangle         barRect = new Rectangle();
    /** The Rectangle of the scale */
    private Rectangle           scaleRect = new Rectangle();
    /** The scale selector  */
    protected boolean           flipscale;
    /** Invers scaling  */
    protected boolean           invers;
    /** The orientation  */
    protected int               orientation = 0;
    /** Buffered image for drawing the scale */
    private Image               bgimg = null;
    /** Max pixels for the bar */
    protected int               maxpixels = 10;
    /** Extend the bar into the scale */
    private int                 extendbar;
    /** Length of the major ticks scale lines */
    protected int               scalelen;
    /** The inset x of the bar, must be changable for sliders */
    protected int               insetx = DEFAULT_INS;
    /** The inset y of the bar, must be changable for sliders */
    protected int               insety = DEFAULT_INS;
    
    /** Create a new instance of JVBar
     */
    public JVBar() {
        Dimension size = new Dimension(100, 40);
        setDesignsize(size);
    }

    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        /* recalc and repaint */
        if (
            key.equals("invers") ||
            key.equals("maxpixels") ||
            key.equals("flipscale") ) {
            recalc = true;
            repaint();
        }
    }
    
    /** Paint this component
     *  @param g the graphics context
     */
    public void paintComponent (Graphics g) {

        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        
        /* Recalc the scale and the bar bounds */
        calcScale(g2);
        /* Draw the border */
        drawBorder(g2);
        /* Draw the bar */
        drawBar(g2, actval, null);
        /* Draw the scale */
        drawScale(g2);
    }

    /** Draw the order, if selected
     *  @param g2 the graphics context
     */
    protected void drawBorder (Graphics2D g2) {
        if (drawBorder) {
            border.paintBorder(this, g2, barRect.x - 2, barRect.y - 2, 
                barRect.width + 4, barRect.height + 4);
        }
    }
    
    /** Draw the bar
     *  @param g2 the graphics context
     */
    protected void drawBar(Graphics2D g2, double val, Color col) {
        /* Get the position of the actual value as a factor of the bar length 
         * and calculate the length of the bar */
        double ext = (double) ((orientation == 0) ? barRect.width : barRect.height);
        int len = (int) (ext * calcPosition(val, 0));
        
        if (col != null)
            g2.setColor(col);
        else
            g2.setColor(getColorForVal(len, ext));
            
        /** Horizontal */
        if (orientation == 0) {
            int h = extendbar * scalelen / 100;
            int y = flipscale ? -h : 0;
            /* Invers (right->left) display ? */
            if (!invers)
                g2.fillRect(barRect.x, barRect.y + y, len, barRect.height + h);
            else
                g2.fillRect(barRect.x + barRect.width - len, barRect.y + y, len, barRect.height + h);
        } else {
            int w = extendbar * scalelen / 100;
            int x = flipscale ? -w : 0;
            /* Invers (top down) display ? */
            if (invers)
                g2.fillRect(barRect.x + x, barRect.y, barRect.width + w, len);
            else
                g2.fillRect(barRect.x + x, barRect.y + barRect.height - len, barRect.width + w, len);
        }
    }

    /** Draw the scale
     *  @param g2 the graphics context
     */
    protected void drawScale(Graphics2D g2) {
        if(bgimg == null)
            return;
        g2.drawImage(bgimg, scaleRect.x, scaleRect.y, null);
    }
    
    /** Calc a scale, if neccecary 
     *  @param g the graphics context
     *
     *  TODO: Check, if Graphics2D.transform can reduce the code
     *  significant. If not, let it in this way, as it is understandable
     */
    protected void calcScale(Graphics g) {

        /* Fill the background, if opaque is true */
        Dimension size = getSize();
        if (isOpaque()) {
            g.setColor(getBackground());
            g.fillRect(0, 0, size.width, size.height);
        }

        /* Nothing to calc, return */
        if (!recalc)
            return;
        recalc = false;
        
        size = new Dimension(getPreferredSize());
        
        /* Preset and take border into concern */
        barRect.setLocation(insetx,insety);
        barRect.width = size.width - 2 * insetx;
        barRect.height = size.height - 2 * insety;
        
        int maxp = (int) (zoomfactor * (double) maxpixels);
        
        if (orientation == 0 && barRect.height > maxp)
            barRect.height = maxp;
        if (orientation == 1 && barRect.width > maxp)
            barRect.width = maxp;

        /* No scale required, return */
        if (majortics <= 0) {
            bgimg = null;
            scalelen = 0;
            return;
        }

        /* retrieve textsize */
        g.setFont(getFont());
        scalelen = g.getFontMetrics().getHeight();
        
        /* Find the value with the maximum text width */
        int wmax = 0;
        for (int i = 0; i <= majortics; i++) {
            int w = g.getFontMetrics().stringWidth(calcMajorText(i));
            if (w > wmax)
                wmax = w;
        }
        
        /* Recalculate the constraints for the bar and the scale */
        scaleRect.setLocation(0,0);
        scaleRect.setSize(size.getSize());
        
        /* Subtract the maximum text width anyway, as we want a symetric display
         * for horizontal
         */
        /* Horizontal or vertical ? */
        if (orientation == 0) {
            barRect.width -= wmax;
            /* add max value half with to the start position of the bar */
            barRect.x += wmax / 2;
            /* if the scale is on top, move the y position of the bar, else
             * move the y position of the scale
             */
            if (flipscale) {
                scaleRect.y = barRect.y;
                barRect.y += 2 * scalelen;
            } else
                scaleRect.y = barRect.y + barRect.height;
            scaleRect.height = 2 * scalelen;
        } else {
            /* add text height to the start position vertical */
            barRect.y += scalelen;
            /* reduce the bar height by 2 * text height */
            barRect.height -= 2*scalelen;
            /* If the scale is on the left side, add text width and 2* text
             * height to the start point x of the bar, else move the
             * start position of the scale
             */
            if (flipscale) {
                scaleRect.x = barRect.x;
                barRect.x += wmax + 2 * scalelen;
            } else
                scaleRect.x = barRect.x + barRect.width;
            scaleRect.width = wmax + 2 * scalelen;
        }
        
        BufferedImage bufimg = new BufferedImage(scaleRect.width, scaleRect.height, BufferedImage.TYPE_INT_ARGB);
        
        /** Get the graphic context */
        Graphics2D g2 = (Graphics2D) bufimg.getGraphics();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

        /* Fill the area transparent */
        g2.setColor(new Color(0, true));
        g2.fillRect(0, 0, scaleRect.width, scaleRect.height);
        
        g2.setFont(getFont());
        
        /* Now draw the scale */
        g2.setColor(getForeground());
        /* Horizontal display ? */
        if (orientation == 0) {
            int x = invers ? barRect.x + barRect.width - 1 : barRect.x;
            int y = flipscale ? scaleRect.height - 3 : 2;
            int len = invers ? -barRect.width : barRect.width;
            g2.drawLine(x, y, x + len, y);
            int ticlen = flipscale ? 2 - scalelen : scalelen - 2;
            for (int i = 0; i <= majortics; i++) {
                int pos = x + (int) (calcMajorTick(i, false) * (double) len);
                g2.drawLine(pos, y, pos, y + ticlen);
                if (i != majortics) {
                    for (int j = 1; j < minortics; j++) {
                        int mpos = x + (int) (calcMinorTick(i, j, false) * (double) len);
                        g2.drawLine(mpos, y, mpos, y + ticlen / 2);
                    }
                }
                String txt = calcMajorText(i);
                int w = g.getFontMetrics().stringWidth(txt);
                int fonty = flipscale ? 
                        y - scalelen + g.getFontMetrics().getDescent()  : 
                        y + 2 * scalelen - g.getFontMetrics().getDescent();
                g2.drawString(txt, pos - w/2, fonty);
            }
        } else {
            int x = flipscale ? scaleRect.width - 3 : 2;
            int y = invers ? barRect.y : barRect.y + barRect.height - 1;
            int len = invers ? barRect.height : -barRect.height;
            g2.drawLine(x, y, x, y + len);
            int ticlen = flipscale ? 2 - scalelen : scalelen - 2;
            for (int i = 0; i <= majortics; i++) {
                int pos = y + (int) (calcMajorTick(i, false) * (double) len);
                g2.drawLine(x, pos, x + ticlen, pos);
                if (i != majortics) {
                    for (int j = 1; j < minortics; j++) {
                        int mpos = y + (int) (calcMinorTick(i, j, false) * (double) len);
                        g2.drawLine(x, mpos, x + ticlen / 2, mpos);
                    }
                }
                String txt = calcMajorText(i);
                int w = g.getFontMetrics().stringWidth(txt);
                int fontx = flipscale ? 
                    x - scalelen - w : x + scalelen;
                int fonty = pos + (scalelen - g.getFontMetrics().getDescent()) / 2;
                g2.drawString(txt, fontx, fonty);
            }
        }
        
        bgimg = bufimg.getScaledInstance(scaleRect.width, scaleRect.height, Image.SCALE_DEFAULT);
       
    }
        
    /** Get the bar oridentation
     * @return bar orientation (HORZIONTAL / VERTICAL)
     */
    public int getOrientation() {
        return orientation;
    }
    
    /** Set the bar orientaiton
     * @param or new bar orientation (HORZIONTAL / VERTICAL)
     */
    public void setOrientation(int or) {
        int old = orientation;
        orientation = or;
        firePropertyChange("orientation", old, orientation);
        if (old != or) {
            Dimension size = new Dimension(getDesignsize());
            int w = size.height;
            size.height = size.width;
            size.width = w;
            setDesignsize(size);
        }
    }

    /** Get the selection of a flipped scale (left, top)
     * @return true if flipped scale is selected
     */
    public boolean isFlipscale() {
        return flipscale;
    }
    
    /** set the selection of a flipped scale (left, top)
     * @param flip true if flipped scale is selected
     */
    public void setFlipscale(boolean flip) {
        boolean old = flipscale;
        flipscale = flip;
        firePropertyChange ("flipscale", old, flip);
    }
    
    /** Get the selection of invers scale
     * @return true, if scaling is invers
     */
    public boolean isInvers() {
        return invers;
    }
    
    /** Set teh selection of a invers scale
     * @param inv true, if scaling is invers
     */
    public void setInvers(boolean inv) {
        boolean old = invers;
        invers = inv;
        firePropertyChange ("invers", old, inv);
    }
    
    /** Get the maximum pixel width/height of the bars short side 
     * @return number of pixels
     */
    public int getMaxpixels() {
        return maxpixels;
    }
    
    /** Set the maximum pixel width/height of the bars short side 
     * @param max number of pixels
     */
    public void setMaxpixels(int max) {
        int old = maxpixels;
        maxpixels = max;
        firePropertyChange ("maxpixels", old, max);
    }
    
    /** Get the setting, to which size the bar should be extended into the scale
     * @return the extension of the bar into the scale (percent of scale tick lenght)
     */
    public int getExtendbar() {
        return extendbar;
    }
    
    /** Set to which size the bar should be extended into the scale
     * @param ext the extension of the bar into the scale (percent of scale tick lenght)
     */
    public void setExtendbar(int ext) {
        int old = extendbar;
        extendbar = ext;
        firePropertyChange ("extendbar", old, ext);
    }

    /** Main method for standalone testing */
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        JPanel p = new JPanel();
        p.setBackground(Color.gray);
        JVBar lvl = new JVBar();
        lvl.setCommVar("t1");
        lvl.setServer("localhost");
        lvl.setCommtype("org.jvisu.comm.JVSimRandomComm");
        lvl.setMaxVar("1");
        lvl.setMinVar("0");
        lvl.majortics = 5;
        lvl.minortics = 10;
        lvl.setFracDigits(2);
        lvl.minalarm = 20;
        lvl.maxalarm = 80;
        lvl.orientation = 0;
        lvl.setDesignsize(new Dimension(500,500));
        lvl.invers = false;
        lvl.flipscale = false;
        lvl.extendbar = 0;
        lvl.setForeground(Color.BLUE);
        lvl.setLogarithmic(false);
        lvl.setFont(new Font("Default", 0, 11));
        lvl.setOpaque(true);
        lvl.drawBorder = true;
        p.add(lvl);
        frame.getContentPane().add(p);
        frame.pack();
        frame.show();
    }    
    
}
