package org.jvisu.base;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.panel.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 *
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * Level widget. It is used to display volumes. The widget uses
 * either the parents background image or an own image.
 * Volumecolor identifies the color of the area which represents
 * the fill volume.	
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVLevel extends JVDouble {
    
    /** color for filllevel */
    private Color               fillColor = Color.green;
    /** color for background */
    private Color               emptyColor = Color.gray;
    /** color for finding the volume */
    private Color               volumeColor = new Color(0xFFAFAF);
    /** Scaled Image for status false */
    private Image               scaledimg;
    /** String representing the location of the image file */
    private String              image;
    /** Use parents background image */
    private boolean             useParentImage;    
    /** Image icon */
    private Image               imgicon;
    /** Buffered image for calculating stuff */
    private BufferedImage       bufimg = null;
    private BufferedImage       drwimg = null;
    /** Number of pixels for this volume */
    private int                 volumePixels = 0;
    /** Pixelbuffer */
    private int[]               pixbuf;
    
    /** Create a new instance of JVLevel
     */
    public JVLevel() {
        this.setDesignsize(new Dimension(100,200));
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
       /* When we use the parents backgorund image, then react on
        * moving and resizing
        */
       if ((key.equals("preferredSize") || key.equals("designpos")) && useParentImage) {
            bufimg = null;
            repaint();
       }
       if (key.compareTo("base") == 0) {
           imgicon = null;
           repaint();
       }
    }

    /** Calculate the number of pixels, which correspond to this volume */
    private int CalcVolumePixels () {
        try {
            return (int)(actval * ((double) volumePixels) / maxval);
        } catch (Exception e) {
            return 0;
        }
    }

    /** VisuDigits specific paint function.
     *  Check, if a min/max check is given<br>
     *  If yes, check, if value is inside the tresholds.<br>
     *  If inside use backcolor, else use alarmcolor
     * @param g Graphics context
     */
    public void paintComponent(Graphics g) {

        /* Draw frame, if configured and get the drawing rectangle */
        Rectangle rect = new Rectangle(getSize());
        
        /* Should we use the parent image ? */
        if (!useParentImage) {
            if (imgicon == null) {
                try {
                    URL url;
                    if (base == null)
                        url = new URL (image);
                    else
                        url = new URL (new URL(base) , image);
                    imgicon = javax.imageio.ImageIO.read(url);
                } catch (Exception e) {return;}
                /* Force reload */
                bufimg = null;
            }
        }
        
        int vcol = volumeColor.getRGB();

        // Must we load an image ? */
        if (bufimg == null || bufimg.getHeight() != rect.height || bufimg.getWidth() != rect.width) {
            /* Should we use the parent image ? */
            if (useParentImage) {
                try {
                    JVPanel panel = (JVPanel) getParent();
                    bufimg = panel.getComponentImage(this);
                    if (bufimg == null)
                        return;
                } catch (ClassCastException ec) {
                    return;
                }
            } else { 
                bufimg = new BufferedImage(rect.width, rect.height, java.awt.image.BufferedImage.TYPE_INT_ARGB);
                Graphics g1 = bufimg.getGraphics();
                g1.drawImage(imgicon.getScaledInstance(rect.width, rect.height, java.awt.Image.SCALE_FAST),0,0,this);
            }
            drwimg = new BufferedImage(rect.width, rect.height, java.awt.image.BufferedImage.TYPE_INT_ARGB);
            pixbuf = new int[rect.height * rect.width];
            bufimg.getRGB(0,0,bufimg.getWidth(), bufimg.getHeight(), pixbuf, 0, bufimg.getWidth());
            volumePixels = 0;
            for (int i = 0; i < pixbuf.length; i++) {
                if (pixbuf[i] == vcol)
                    volumePixels++;
            }
        } 
        
        // calc volumepixels
        int actPixels = CalcVolumePixels();
        // get color RGB values
        int fcol = fillColor.getRGB();
        int bcol = emptyColor.getRGB();
        
        bufimg.getRGB(0,0,bufimg.getWidth(), bufimg.getHeight(), pixbuf, 0, bufimg.getWidth());
        for (int i = pixbuf.length - 1; i >= 0; i-= rect.width) {
            for (int j = 0; j < rect.width; j++) {
                if (pixbuf[i-j] == vcol) {
                    pixbuf[i-j] = (actPixels > 0) ? fcol : bcol;
                    actPixels--;
                }
            }
        }
        drwimg.setRGB(0,0,rect.width, rect.height, pixbuf, 0, rect.width);
        
        g.drawImage(drwimg, rect.x, rect.y, null);
    }
    
    /** Getter for property image.
     * @return Value of property image.
     *
     */
    public String getImage() {
        return this.image;
    }
    
    /** Setter for property image.
     * @param image New value of property image.
     *
     */
    public void setImage(String image) {
        this.image = image;
        imgicon = null;
        repaint();
    }
    
    /** Getter for property emptyColor.
     * @return Value of property emptyColor.
     *
     */
    public Color getEmptyColor() {
        return this.emptyColor;
    }
    
    /** Setter for property emptyColor.
     * @param emptyColor New value of property emptyColor.
     *
     */
    public void setEmptyColor(Color emptyColor) {
        this.emptyColor = emptyColor;
        repaint();
    }
    
    /** Getter for property fillColor.
     * @return Value of property fillColor.
     *
     */
    public Color getFillColor() {
        return this.fillColor;
    }
    
    /** Setter for property fillColor.
     * @param fillColor New value of property fillColor.
     *
     */
    public void setFillColor(Color fillColor) {
        this.fillColor = fillColor;
        repaint();
    }
    
    /** Return if the parent image should be used
     * @return true, if parent image should be used
     */
    public boolean isUseParentImage() {
        return this.useParentImage;
    }
    
    /** Set, if the parent image should be used by this component
     * @param useParentImage true, if parent image should be used
     */
    public void setUseParentImage(boolean useParentImage) {
        this.useParentImage = useParentImage;
        repaint ();
    }

    /** Get the color, which is used to identify the volume area inside 
     *  the image.
     * @return volume indentification color.
     */
    public Color getVolumeColor() {
        return this.volumeColor;
    }
    
    /** Set the color, which is used to identify the volume area inside 
     *  the image.
     * @param volumeColor New volume identification color.
     */
    public void setVolumeColor(Color volumeColor) {
        this.volumeColor = volumeColor;
        repaint();
    }

    /** Main method for standalone testing */
    /*
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        JVLevel lvl = new JVLevel();
        lvl.setCommVar("t1");
        lvl.setMaxVar("10");
        lvl.setMinVar("1");
        lvl.setServer("localhost");
        lvl.setCommtype("org.jvisu.comm.JVSimRandomComm");
        lvl.setImage("file:///home/thomas/projekte/Maass/test/www/images/silo2.gif");
        frame.getContentPane().add(lvl);
        frame.pack();
        frame.show();
    }    
    */
}
