package org.jvisu.comm;

import java.beans.*;
import java.util.*;
import org.jvisu.comm.var.*;

/** Abstract base class for JVisu communication.
 *
 *  <p>$Id: JVComm.java,v 1.15 2004/04/19 06:52:46 tgl Exp $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplementation </li>
 *  </ul>
 *  <p>
 *  <p>This abstract class provides the communication functions for JVisu.
 *  If the existing implementations do not fit your needs, implement your own.
 *  based on this. 
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
 * @since JVisu 1.0
*/
public abstract class JVComm extends Object {

    /** Arraylist of registered variables, which should be retrieved from the server */    
    protected ArrayList         vars;
    /** Configuration of this communication instance */
    protected JVCommCfg         commcfg;
    /** Connection status */
    private boolean             connected;
    /** Semaphore object for concurrency control in the instance context*/
    protected Object            instsem = new Object();
    /** status */
    protected int               status;

    /** Static ArrayList to store the available instances of JVisu communication */    
    private static ArrayList    instances = new ArrayList();
    /** Semaphore object for concurrency control in the static context*/
    private static Object       sem = new Object();
    /** Status logging */
    private static ArrayList    statusText = new ArrayList();
    /** status */
    private static boolean      statusOK = false;
    /** Maximum status messages stored */
    public static final int    MAX_STATUS_MSG = 50;
    /** Property change support */
    private static PropertyChangeSupport changeSupport = 
            new PropertyChangeSupport(JVComm.class);
    
    /** Static string array, which contains the registered class names of
     *  JVisu communication classes. The class must not be available at runtime
     *  except those, which are used in the application.
     *  This list is mainly used to determine the actual available communication
     *  classes for the JVPanelConfigurator RAD tool
     *  Set the full classname here !
     */
    private static String[][] commtypes = {
        {"org.jvisu.comm.JVSimComm","Simulation"},
        {"org.jvisu.comm.JVSimRandomComm","Random generator"},
        {"org.jvisu.comm.JVSockComm","JVisu socket"},
        {"org.jvisu.comm.JVAcpltComm", "ACPLT connection"},
    };

    /** Create a new instance of JVComm
     *  build a new variable list 
     */
    public JVComm() {
        vars = new ArrayList();
    }

    /** Check, if the given configuration is valid for
     * this communication type
     * @param cfg the configuration to check
     * @return true, if valid
     */
    protected abstract boolean isValid(JVCommCfg cfg);

    /** Send a variable value to the server
     *  @param var JVisu variable which has to be sent to the server
     *  @return true on success
     */    
    protected abstract boolean SendVar (JVVar var);
    
    /** Start communication to server.
     */    
    protected abstract void startComm ();

    /** Stop communication to server.
     * This can only be done, if all variables are unregistered.
     */    
    protected abstract boolean stopComm ();

    /** Check, if the instance is matching
     * @return true, if matching
     */    
    protected abstract boolean isMatching (JVCommCfg cfg);

    /** Check, if the instance is reusable
     * @return true, if reusable
     */    
    protected abstract boolean isReusable (JVCommCfg cfg);

    /** Return the configuration of this instance
     *  @return configuration
     */
    protected final JVCommCfg getCfg() {
        return commcfg;
    }

    /** Register a JVisu variable with this instance. The functionality of
     *  this method depends on the communication type. A simulation instance
     *  does nothing and a real connecting instance may use this method for
     *  registering vars in the communication server or other functionality.
     *  This method is called from a synchronized context and must not
     *  be synchronized seperately.
     *  @param var JVisu variable, which has to be registered
     */    
    protected abstract void RegisterVar (JVVar var);

    /** Unregister a JVisu variable in the comm instance. The functionality
     *  of this method depends on the communication instance type.
     *  This method is called from a synchronized context and must not
     *  be synchronized seperately.
     * @param var JVisu variable, which has to be unregistered
     */    
    protected abstract void UnRegisterVar (JVVar var);

    /** Set the configuration of this instance
     *  @param cfg configuration of this instance
     */
    private final void setCfg(JVCommCfg cfg) throws IllegalArgumentException {
        if (!isValid(cfg))
            throw new IllegalArgumentException();
        commcfg = new JVCommCfg(cfg);
    }

    /** Check, if this comminstance has a registered variable, 
     *  which matches the given variable configuration
     *  This method is called from a synchronized context and must not
     *  be synchronized seperately.
     *  @param cfg variable configuration to seek for
     *  @return variable instance, if a matching variable is available, else null
     */
    protected final JVVar findVariable (JVVarCfg cfg) {
        for (int i = 0; i < vars.size(); i++) {
            JVVar v = (JVVar) vars.get(i);
            if (v.getVarCfg().isMatching(cfg)) {
                return v;
            }
        }
        return null;
    }

    /** Check, if this comminstance has a registered variable, 
     *  which matches the given symbol
     *  This method is called from a synchronized context and must not
     *  be synchronized seperately.
     *  @param symbol variable symbol to seek for
     *  @return variable instance, if a matching variable is available, else null
     */
    protected final JVVar findVariable (String symbol) {
        for (int i = 0; i < vars.size(); i++) {
            JVVar v = (JVVar) vars.get(i);
            if (v.getVarCfg().getSymbol().equals(symbol)) {
                return v;
            }
        }
        return null;
    }
    
    /** Register JVisu variables.
     *  If a variable with a given configuration is already available,
     *  then the listener is added to this variable.
     *  If not, a new variable of the given type is created, added
     *  to the variable array and the listener is added to the variable
     *
     * @param cfg variable configuration array
     * @param listener the variable listener, which wants to listening on 
     *        the variable(s)
     */    
    private final void RegisterVars (JVVarCfg[] cfg, JVVarListener listener) {
        /* Must be synchronized inside the instance */
        synchronized (instsem) {
            for (int i = 0; i < cfg.length; i++) {
                RegisterVar(cfg[i], listener);
            }
        }
    }

    /** Register JVisu variables.
     *  If a variable with a given configuration is already available,
     *  then the listener is added to this variable.
     *  If not, a new variable of the given type is created, added
     *  to the variable array and the listener is added to the variable
     *
     * @param cfg variable configuration array
     * @param listener the variable listener, which wants to listening on 
     *        the variable(s)
     */    
    private final void RegisterVar (JVVarCfg cfg, JVVarListener listener) {
        /* Do not register disabled and empty variables */
        if (!cfg.isEnabled() || cfg.getSymbol() == null || cfg.getSymbol().length() == 0)
            return;
        /* Try to find a existing variable */
        JVVar var = findVariable(cfg);
        if (var != null) {
            /* Add the listener to the variable */
            var.addVarListener(listener);
        } else {
            /* Create a new variable instance */
            var = JVVar.CreateInstance (cfg, listener);
            if (var != null) {
                vars.add (var);
                RegisterVar (var);
            }
        }    
        cfg.setVarInstance(var);
    }
    
    /** Unregister JVisu variables.
     *  This removes the variable listener and removes the variable from
     *  the variable array, if the variable has no more active listeners
     *  @param cfg variable configuration array to find a matching instances
     *  @param listener the variable listener, which was listening on this variable
     */    
    private final void UnRegisterVar (JVVarCfg cfg, JVVarListener listener) {
        synchronized (instsem) {
            JVVar var = cfg.getVarInstance();
            if (var != null) {
                var.delVarListener(listener);
                /* Check, if the listeners list is empty */
                if (var.isRemovable()) {
                    /* Unregister in the communication instance */
                    UnRegisterVar (var);
                    /* Remove from the list */
                    vars.remove(var);
                }
            }
            cfg.setVarInstance(null);
        }
    }

    /** Get a variable value.
     *  Lookup the variable instance by comparing the configuration and 
     *  return the value as String
     *  @param symbol the symbol to look up
     *  @return variable value as String, if variable is found else null
     */
    public String getVarValue (String symbol) {
        synchronized (instsem) {
            JVVar var = findVariable(symbol);
            if (var != null)
                return var.toString();
        }
        return null;
    }

    /** Set a variable value.
     *  Lookup the variable instance by comparing the configuration and 
     *  set the value from the given String
     *  @param symbol the symbol to look up
     *  @param value  the value as String
     */
    public void setVarValue (String symbol, String value) {
        synchronized (instsem) {
            JVVar var = findVariable(symbol);
            if (var == null)
                 return;
            if (var.SetVarValue(value))
                 this.SendVar(var);
        }
    }

    /** Set a variable value.
     *  Lookup the variable instance by comparing the configuration and 
     *  set the value from the given object
     *  @param cfg The variable config to lookup
     *  @param value  the value as object
     */
    public final void setVarValue (JVVarCfg cfg, Object obj) {
        synchronized (instsem) {
            JVVar var = cfg.getVarInstance();
            if (var != null && var.SetVarValue(obj))
                this.SendVar(var);
        }
    }
    
    /** Get the connection status
     * @return connection status
     *
     */
    public final boolean isConnected() {
        return connected;
    }
    
    /** Set the connection status
     * @param status The connection status
     */
    protected void setConnected(boolean status) {
        boolean oldConnected = this.connected;
        this.connected = status;
    }
    
    /** Find a variable, where serverindex is equal to the given index.
     * @param serverIndex Index of the variable, when registered with server
     * @return matching variable or null, if none found
     */
    protected JVVar GetVariable (int serverIndex) {
        synchronized (instsem) {
            for (int i = 0; i < vars.size(); i++) {
                JVVar v = (JVVar) vars.get(i);
                if (v.getSrvIndex() == serverIndex)
                    return v;
            }
        }
        return null;
    }
    
    /** Return the registered communication types
     *  @return the description array
     */
    public static String[][] getCommTypes() {
        return commtypes;
    }
    
    /** Get the variable value for a given server, port and symbol.
     *  Used for scripting
     *  @param cfg      Comm configuration
     *  @param symbol   the variable symbol
     */
    public static synchronized String getVarValue(JVCommCfg cfg, String symbol) {
        synchronized (sem) {
            for (int i = 0; i < instances.size(); i++) {
                JVComm inst = (JVComm) instances.get(i);
                if (inst.isMatching(cfg))
                    return inst.getVarValue(symbol);
            }
        }
        return null;
    }
    
    /** Set the variable value for a given server, port and symbol.
     *  Used for scripting
     *  @param cfg      Comm configuration
     *  @param symbol   the variable symbol
     *  @param value    the variable value
     */
    public static synchronized void setVarValue(JVCommCfg cfg, String symbol, String value) {
        synchronized (sem) {
            for (int i = 0; i < instances.size(); i++) {
                JVComm inst = (JVComm) instances.get(i);
                if (inst.isMatching(cfg)) {
                    inst.setVarValue(symbol, value);
                    return;
                }
            }
        }
    }
    
    /** Register a set of variable configurations in a communication instance
     * if possible
     * @param cfg communication configuration
     * @param varcfg variable configuration array
     * @param listener the variable listener, which wants to listen on 
     *        the variable(s). Usually the calling component.
     */    
    public final static void RegisterVariables (JVCommCfg cfg, JVVarCfg[] varcfg, JVVarListener listener) {
        
        boolean varvalid = false;
        
        /* Check, if there is a valid variable configuration
         */
        for (int i = 0; i < varcfg.length; i++) {
            if (varcfg[i].isEnabled() && varcfg[i].getSymbol() != null 
                    && varcfg[i].getSymbol().length() != 0) {
                varvalid = true;
            }
        }
        
        if (varvalid == false)
            return;

        JVComm inst;
        /* This must be done atomic ! */
        synchronized (sem) {
            /* Get a communication instance. 
             * Either an existing matching one or a new one 
             */
            inst = GetInstance (cfg);
            cfg.setCommInstance(inst);
            if (inst == null)
                return;
        }
        /* The registering is done atomic inside the instance */
        inst.RegisterVars(varcfg, listener);
    }

    /** Unregister a set of variable configurations in a communication instance
     * @param commcfg the communication configuration
     * @param varcfg variable configuration array
     * @param listener the variable listener, which wants to listen on 
     *        the variable(s). Usually the calling component.
     */    
    public final static void UnRegisterVariables(JVCommCfg commcfg, 
                            JVVarCfg[] varcfg, JVVarListener listener) {
        
        synchronized (sem) {
            JVComm inst = commcfg.getCommInstance();
            if (inst == null)
                return;
            for (int i = 0; i < varcfg.length; i++) {
                inst.UnRegisterVar(varcfg[i], listener);
            }
            commcfg.setCommInstance(null);
            // Check, if the instance can be shut down
            ReleaseInstance(inst);
        }
    }
    
    /** Update variable configuration data
     *  If the variable is registered in a communication instance, unregister the 
     *  variable and register the new configuration again, if possible
     * @param commcfg the communication configuration
     * @param oldcfg old variable configuration
     * @param varcfg new variable configuration
     * @param listener the variable listener, which wants to listen on 
     *        the variable(s). Usually the calling component.
     */    
    public final static void UpdateVarCfg (JVCommCfg commcfg, JVVarCfg oldcfg, 
                JVVarCfg newcfg, JVVarListener listener) {
        
        synchronized (sem) {
            JVComm inst = commcfg.getCommInstance();
            if (inst != null) {
                inst.UnRegisterVar(oldcfg, listener);
                inst.RegisterVar(newcfg, listener);
                if (newcfg.getVarInstance() == null && ReleaseInstance(inst) != null)
                        commcfg.setCommInstance(null);
            } else {
                JVVarCfg[] cfg = { newcfg };
                RegisterVariables(commcfg, cfg, listener);
            }
        }
    }
    
    /** Update communication configuration data
     * @param oldcfg the old communication configuration
     * @param newcfg the new communication configuration
     * @param varcfg variable configuration array
     * @param listener the variable listener, which wants to listen on 
     *        the variable(s). Usually the calling component.
     */    
    public final static synchronized void UpdateCommCfg (JVCommCfg oldcfg, 
                JVCommCfg newcfg, JVVarCfg[] varcfg, JVVarListener listener) {
        
        synchronized (sem) {
            JVComm inst = oldcfg.getCommInstance();
            if (inst != null) {
                for (int i = 0; i < varcfg.length; i++)
                    inst.UnRegisterVar(varcfg[i], listener);
                /* Remove the reference */
                oldcfg.setCommInstance(null);
                /* Check, if this instance can be released */
                ReleaseInstance(inst);
            }
        }
        RegisterVariables(newcfg, varcfg, listener);
    }
    
    /** Get an existing or create a new JVisu communication instance.
     * Check, if there is a matching instance available. If yes use this, else
     * create a new one with the given configuration
     *  This method must be called with the global semaphore held.
     * @param cfg configuration, which has to be compared
     * @param inst the instance, which is currently used by this component
     * @param varcfg variable configuration array
     * @param listener the variable listener, which wants to listening on 
     *        the variable(s)
     * @return An existing or a newly created instance of JVisu communication
     */    
    private static JVComm GetInstance (JVCommCfg cfg) {
        /* walk through the list and find a matching instance */
        for (int i = 0; i < instances.size(); i++) {
            JVComm inst = (JVComm) instances.get(i);
            /* Check, if the communication instance can be reused */
            if (inst.isMatching(cfg))
                return inst;    
        }
        
        /* Ok create a new one */
        try {           
            Class classDef = Class.forName(cfg.getType());
            Object obj = classDef.newInstance();
            JVComm inst = (JVComm) obj;
            inst.setCfg(cfg);
            inst.startComm();
            instances.add(inst);
            return inst;
        } catch (InstantiationException e) {
            System.out.println(e);
        } catch (IllegalAccessException e) {
            System.out.println(e);
        } catch (ClassNotFoundException e) {
            System.out.println(e);
        } catch (ClassCastException e) {
            System.out.println(e);
        } catch (IllegalArgumentException e) {
            // This is reported in the status
            return null;
        } catch (Exception e) {
            System.out.println(e);
        }
        return null;
    }
    
    /** Release a communication instance, if possible. The release can only
     *  be done, if the instance has no more registered variables
     *  This method must be called with the global semaphore held.
     *  @param inst the instance, which should be released
     *  @return null, if the instance can't be released, else the instance
     *          which was released and removed from the list    
     */
    private static JVComm ReleaseInstance (JVComm inst) {
        if (inst != null) {
            if (inst.vars.size() > 0) 
                return null;
            if (inst.stopComm()) {
                instances.remove(inst);
                setStatusMessage (inst, 0, "Remove instance");
                return inst;
            }
        }
        return null;
    }
    
    /** Return the status */
    public final static boolean isStatusOK () {
        return statusOK;
    }
    
    /** Return the status message array */
    public final static String[] getStatusMessages() {
        String txt[] = new String[statusText.size()];
        return (String[]) statusText.toArray(txt);
    }
    
    /** Set a status message 
     *  @param inst the JVComm instance, which is producing this message
     *  @param stat the numerical status representation
     *  @param msg the message string for this entry
     */
    protected final static void setStatusMessage (JVComm inst, int stat, String msg) {
        if (statusText.size() == MAX_STATUS_MSG)
            statusText.remove(MAX_STATUS_MSG - 1);
        inst.status = stat;
        String txt = (new java.util.Date(System.currentTimeMillis())).toString();
        JVCommCfg cfg = inst.getCfg();
        if (cfg != null)
            txt += " : " + cfg.getType() + " : " + cfg.getServer() + " : " + cfg.getPort();
        else
            txt += " : " + inst.getClass().toString();
        txt += " : " + msg;
        statusText.add(0, txt);
        changeSupport.firePropertyChange("statusmsg", null, txt);
        //System.out.println(msg + ":" + stat);
        boolean newstat = true;
        for (int i = 0; i < instances.size(); i++) {
            if (((JVComm) instances.get(i)).status != 0)
                newstat = false;
        }
        setStatus(newstat);
    }

    private static void setStatus (boolean stat) {
            boolean old = statusOK;
            statusOK = stat;
            changeSupport.firePropertyChange("commstatus", old, stat);
    }
    
    /** Add a status change listener
     */
    public static void addPropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.addPropertyChangeListener(listener);
    }

    /** Remove a status change listener
     */
    public static void removePropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.removePropertyChangeListener(listener);
    }
}
